#!/usr/local/bin/perl

# muppet.cgi by Sam Nelson, February , 1996, v1.0
# A CGI to implement a treasure search game
# 
# Requires the library cgi-lib.pl.  
# Requires a directory called $DATADIR (see below) containing several 
# sets of files:
#
# -- clue.$stage.$substage
#    Clue for a given stage and substage.
# -- clue.$stage.$substage.image
#    The filename for the image associated with the clue.
# -- finished.$stage.$substage
#    The message the user gets when finished with a given substage.
# -- finished.$stage.$substage.image
#    The image filename associated with the above.
# -- generic.$stage.$substage.$num
#    Text for a generic clue for a given substage.
# -- generic.$stage.$substage.$num.image
#    The image filename associated with above.
# -- hotspot.$stage.$substage.$num
#    Text for a hotspot for a given substage.
# -- hotspot.$stage.$substage.$num.image
#    The image filename associated with above.
# -- im.$stage.$substage
#    The imagemap associated with a given substage.
# -- image.$stage
#    The filename of the main graphic for a stage .
#    (im.$stage.$substage overlays this file.)
# -- welcome.$stage
#    Text of the welcome message associated with a stage.
# -- welcome.$stage.image
#    The image associated with this welcome message.
# -- The files pointed to in all of the *.image and image.* above.
#    Can be .gif, .jpeg, or whatever.
#

# --------------------------------------------------------------------
# Constants, requires, and other good stuff
# --------------------------------------------------------------------

#
# Start output to CGI
#
require 'cgi-lib.perl4.pl';
&ReadParse();

#
# Constants
#
$BADTOKENFILE="Bad Token File";
$MAXHOTSPOTS=20;
$DATADIR="./data/";
$DEFAULTTITLE="Muppet Treasure Hunt";

#
# Hardwired data -- Seth _promised_ we wouldn't have to change this!
# He promised!
#
$numstages=4;
$numsubstages[1]=4;
$numsubstages[2]=4;
$numsubstages[3]=5;
$numsubstages[4]=1;

# --------------------------------------------------------------------
# Get all necessary data from $in{} or otherwise
# --------------------------------------------------------------------

#
# $stage and $substage contain the number of the current stage/substage
#
($stage=$in{"stage"}) || ($stage=1);
($substage=$in{"substage"}) || ($substage=1);

#
# @clues is a space-separated list of clue codes. Codes are in the order
# received.  Clues look like 
# [clue|generic|hotspot].$stage.$substage[.$num], i.e.:
#   clue.2.3  -- the main clue for stage 2, substage 3
#   generic.1.4.2 -- the second generic clue for stage 1, substage 4
#   hotspot.1.2.3 -- the third hotspot clue for stage 1, substage 2
#
(@clues=split(" ",$in{"clues"}));

#
# $cookie is 0 for new arrivals, 1 for old visitors
#
($cookie=$in{"cookie"}) || ($cookie=0);

#
# $score is the total number of clicks it took.
#
($score=$in{"score"}) || ($score=0);

#
# $GL is the graphics level, with a value of either "L" or "H".
#
($GL=$in{"GL"}) || ($GL="H");

#
# $res_x is the x-coordinate of the user click in the resolution-setting image
#
$res_x=$in{"res.x"};

# --------------------------------------------------------------------
# Update all variables, based on game data
# ALL the state changes of the program occur right here!  So pay attention!
# Sorry -- didn't mean to get so abrasive.
# --------------------------------------------------------------------

# Inputs:
# -- $stage:  The previous game stage (integer)
# -- $substage:  The previous game clue within $stage (integer)
# -- $cookie:  1 if we've been to the site before, 0 otherwise (boolean)
# -- @clues:  The previous set of clues on last page (array of string)
# -- &getdata( ... ):  All game data (sorta associative array on filenames)
# -- $numstages:  Number of stages in this game (integer)
# -- @numsubstages:  Number of clues in a given substage (array of integer)
# -- $score:  The total number of clicks so far
# -- $GL:  The graphics level
# -- $token:  Indication of where the user just clicked.  We take (integer)
#    CORRECT  for a correct hotspot
#    GENERIC  for a generic click
#    #        for a particular incorrect hotspot
#
# Actions (exactly one of the following, in order):
# -- New arrivals to the site
# -- New arrivals to the substage
# -- Generic (non-hotspot) click
# -- Last-stage hotspot click
# -- Other stage hotspot click
# -- Finished stage (correct spot)
# -- Finished substage but not stage (correct spot)
#
# Output (variables modified):
# -- $title:  The title of this page (string)
# -- @clues:  Set of clues available at this page (array of string)
# -- $cookie:  Give a cookie (determines "new arrivals to the site") (boolean)
# -- $message:  If this stage has a message that is not a clue (boolean)
# -- $gameover:  If we're done with the game (string)
# -- $stage:  The current game stage (integer)
# -- $substage:  The current clue within $stage (integer)
# -- $newstage:  Are we starting a new stage right now? (boolean)
# -- $score:  The total number of clicks, incremented for this page 
# -- $GL:  The graphics level
#
# Code is sometimes repeated; for example, both "new arrivals to the site"
# and "finished stage" cases have to set $message to 
# &getdata("welcome.$stage").
#
#
# Here goes: 
#

#
# We might have to force-redirect to disney (top bar, not in graphics
# region)
#

if (($res_x > 0) && ($res_x < 116)){
    
    #
    # Clicked on the bar, not in the resolution area.
    # Go to disney.
    #
    
    &send_redirect ( "http:/" );
    exit(0);
} elsif ($res_x>385) {
    &send_redirect ( "http:/Contents/contents.html\#WALTDISNEYHOMEVIDEO" );
    exit(0);
} elsif ($res_x>116) {
    &send_redirect ( "http:/DisneyVideos/" );
    exit(0);
}


#
# TOO LATE FOR REDIRECTS!!  Start actually processing this page.
#

open ( FILTER_OUT , '| /pool/disney/cgi-bin/perl4_filter -noheader');
if (FILTER_OUT)
{
#    select(FILTER_OUT);
}
print &PrintHeader();

#
# Get the token based on where the user clicked and the image map (im)
# for this stage and clue.
#
$token=&tokenize(&getdata("im.$stage.$substage"));

####################################################
# This code commented out so we no longer have H & L modes
# -- jeff
#
#if ($res_x>385)     # Set the resolution first, then do other stuff
#
#{                    
#
#    if ($GL eq "L")
#    {
#	$GL = "H";
#    } else 
#    {
#	$GL = "L";
#    }
#}
####################################################

 START_PROCESSING:
    #
    # no-op -- for some reason, necessary to make the goto work.
    # Ok, ok, sorry about the goto.  I'll get rid of it later.
    #
    if (0) { }

if (!$cookie)        # -- A new arrival!  Hooray!  Someone's using the site!

{
    undef(@clues);
    $newstage=1;
    $cookie=1;

    $message = &imagetag ( "welcome.image" );
    $message .= "<BR CLEAR>";
    $message .= &getdata("welcome");
    $message .= "<HR>";

    $message .= &imagetag ( "welcome.$stage.image" );
    $message .= "<BR CLEAR>";
    $message .= &getdata("welcome.$stage");
    $message .= "<P><HR><P>\n";

    $score++;
    $title="Welcome to the Muppet Treasure Hunt!";
}

elsif (!$clues[0])          # -- Starting this substage, no clues so far

{
    push(@clues,"clue.$stage.$substage");
    $score++;
    $title="$DEFAULTTITLE -- Map $stage -- Clue $substage -- Score $score";
} 

elsif ($token eq 'GENERIC')       # -- A generic (non-hotspot) click

{
    local ($numgeneric)=(1);
    foreach (@clues) 
    {
	if (/^generic/)
	{
	    ++$numgeneric;
	}
    }
    if(&getdata("generic.$stage.$substage.$numgeneric"))
    {
	push(@clues,"generic.$stage.$substage.$numgeneric");
    } 
    else   # No more generic clues, so on to the next stage
    {  
	$token='CORRECT';

	# 
	# A brutal, evil, and unjustifiable GOTO!!!!
	#

	goto START_PROCESSING;
    }

    $score++;
    $title="$DEFAULTTITLE -- Map $stage -- Clue $substage -- Score $score";
} 

elsif (($token>0) && ($stage==$numstages))  # Game over, man.  
                                            # Last stage has 1 substage
                                            # hotspots do special things
{
    $gameover=1;
    undef(@clues);
    push(@clues,"hotspot.$stage.$substage.$token");
    $title="Game Over -- Score $score";
}

elsif ($token>0)         # -- Clicked on a hotspot (not last stage

{
    local ($allreadyhit)=(0);
    foreach (@clues) {
	if (/^hotspot.$stage.$substage.$token$/){
	    $allreadyhit=1;
	}
    }
    if (!$allreadyhit){
	push(@clues,"hotspot.$stage.$substage.$token");
    }
    
    $score++;
    $title="$DEFAULTTITLE -- Map $stage -- Clue $substage -- Score $score";
}

elsif (($token eq 'CORRECT') && ($substage==$numsubstages[$stage]))  
                       # -- Finished stage
{                          
    if ($m = &getdata("finished.$stage.$substage")){

	$message  = &imagetag( "finished.$stage.$substage.image" );
	$message .= "<BR CLEAR> $m <P><HR><P>\n";
    }

    ++$stage;
    $substage=1;
    undef(@clues);
    $newstage=1;

    if ($stage==$numstages){     # Last stage doesn't have intro page
	$newstage=0;
	push(@clues,"clue.$stage.$substage");
    }  

    if ($m = &getdata("welcome.$stage")){
	
	$message .= &imagetag( "welcome.$stage.image" );
	$message .= "<BR CLEAR> $m <P><HR>\n";
    }

    $score++;
    $title="Congratulations!  You finished the stage!"
	. "-- Map $stage -- Clue $substage -- Score $score";
}

elsif ($token eq 'CORRECT')          # -- finished substage but _not_ stage

{
    undef(@clues);
    if ($m = &getdata("finished.$stage.$substage")){

	$message .= &imagetag( "finished.$stage.$substage.image" );
	$message .= "<BR CLEAR> $m <P><HR>\n";
    }
    $substage++;
    push(@clues,"clue.$stage.$substage");

    $score++;
    $title="Nice job! -- Map $stage -- Clue $substage -- Score $score";
}

# --------------------------------------------------------------------
# Output the HTML page -- all output occurs here
# --------------------------------------------------------------------

&printopening;    # <HTML>,<HEAD>,<TITLE>,<BODY>,<CENTER> tags
&printbar;        # The top bar, which is a form submit button
&printmessage;    # If any $message (finished substage) print it at top

&printformopening;
if ($gameover)    # Game is over
{
    &printcluetable;
}
elsif (!$newstage)   # it's not our first visit to this stage
{    
    &printimagemap;    # The image form submit button
    &printcluetable;   # All clues so far, including icon for first speaker
} 
else      # it _is_ our first visit to this stage; no need for map/clues
{
    &printaltsubmit;   # No map, just a submit button
}
&printformdata;
print "</FORM>";

# commented out by matt rochlin 7/1/96 to remove sweepstakes refs
# if ($gameover){
#     &printregister;
# }
&printsavegame;
&printlegal;
&printclosing;    # Close all HTML tags
exit(0);          # Goodbye, cruel world.

#******************* SUB PRINTBAR ***************************************
#*Print the high-res/lo-res bar across the top
#******************* SUB PRINTBAR ***************************************

sub printbar {
    &printformopening;

    if($GL eq "L"){
	#print "<INPUT TYPE=\"image\" NAME=\"res\" ";
	#print "SRC=\"/DisneyVideos/Images/toolbar_top.gif\" BORDER=\"0\">\n";
    } else {
	$GL="H";
	#print "<INPUT TYPE=\"image\" NAME=\"res\" ";
	#print "SRC=\"/DisneyVideos/Images/toolbar_top.gif\" BORDER=\"0\">\n";
    }
    print "<BR CLEAR>\n";

    foreach $key (keys(%in)){
	if ($key ne "GL"){
	    print "<INPUT TYPE=\"hidden\" NAME=\"$key\" VALUE=\"$in{$key}\">\n";
	}
    }
    print "<INPUT TYPE=\"hidden\" NAME=\"GL\" VALUE=\"$GL\">\n";
    print "</FORM>";
}    
#******************* SUB PRINTREGISTER *********************************
#*Print a self-contained form to let the user register their high score.
#******************* SUB PRINTREGISTER **********************************

sub printregister {
    print "
<FORM METHOD=\"get\" ACTION=\"/MuppetTreasure/hunt-cgi/register.html\">
<INPUT TYPE=\"hidden\" NAME=\"GL\" VALUE=\"$in{'GL'}\">
<INPUT TYPE=\"hidden\" NAME=\"score\" VALUE=\"$score\">
<INPUT TYPE=\"submit\" VALUE=\"Enter the Muppet Treasure Island Sweepstakes\">
</FORM>
";
}

#******************* SUB PRINTSAVEGAME *********************************
#*Print a self-contained form to let the user save the game.
#*If this utiility is to work, it requires a cgi called bookmark_me.cgi
#*in the same directory.
#******************* SUB PRINTSAVEGAME **********************************

sub printsavegame {
    local ($data);
    $data=join("\t",%in);
    $cryptext=pack("u*",$data);
    @cryptext=split(//,$cryptext);
    for($count=0;$count<=$#cryptext;$count++){
	$cryptext[$count]=ord($cryptext[$count]);
    }
    $cryptext=join(" ",@cryptext);
    print "
<FORM METHOD=\"get\" ACTION=\"bookmark_me.cgi\">\n
<INPUT TYPE=\"hidden\" NAME=\"GL\" VALUE=\"$in{'GL'}\">\n
<INPUT TYPE=\"hidden\" NAME=\"data\" VALUE=\"$cryptext\">\n
<INPUT TYPE=\"submit\" VALUE=\"Save Game\">\n
</FORM>\n
";
}

#******************* SUB PRINTMESSAGE *********************************
#*Print a message, if any.  We assume that the message includes any
#*necessary html tags.
#******************* SUB PRINTMESSAGE **********************************

sub printmessage {
    if ($message){
	print "$message";
    }
}

#******************* SUB PRINTALTSUBMIT *********************************
#*Print an alternative submit button
#******************* SUB PRINTALTSUBMIT **********************************

sub printaltsubmit {
    print "<INPUT TYPE=\"submit\" VALUE=\"Onward!\">\n";
}

#******************* SUB PRINTIMAGEMAP *********************************
#*Print the imagemap form input button
#******************* SUB PRINTIMAGEMAP **********************************

sub printimagemap {
    print "<INPUT TYPE=\"image\" NAME=\"map\" SRC=\"$DATADIR" 
	. $GL . "." . &getdata("image.$stage") . "\" BORDER=\"\">\n";
}

#******************* SUB PRINTCLUETABLE *********************************
#*Print the table of clues, including the image of whoever says the
#*first clue.
#******************* SUB PRINTCLUETABLE **********************************

sub printcluetable {
    print "\n<!-- #START TABLES=\"T\" -->\n";
    print "<TABLE WIDTH=100%>";
    print "<TR>";
    print "<TD VALIGN=top ALIGN=letf>\n";
    print "\n<!-- #STOP TABLES=\"T\" -->\n";

    print &imagetag( "$clues[$#clues].image" );
    
    print "\n<!-- #START TABLES=\"T\" -->\n";
    print "</TD>";
    print "<TD VALIGN=top ALIGN=letf>\n";
    print "\n<!-- #STOP TABLES=\"T\" -->\n";
	
    print "<H3>";
    print &getdata($clues[$#clues]);
    print "</H3>";

    print "\n<!-- #START TABLES=\"T\" -->\n";

    print "</TD></TR><TR><TD VALIGN=top ALIGN=letf COLSPAN=2>\n";
    print "\n<!-- #STOP TABLES=\"T\" -->\n";

    if($#clues>0){
	print "<OL>\n";
	for($count=$#clues-1;$count>=0;$count--){
	    print "<LI>";
	    print &getdata($clues[$count]);
	    print "<P>\n";
	}
	print "</OL>\n";
    }

    print "\n<!-- #START TABLES=\"T\" -->\n";
    print "</TD></TR></TABLE>\n";
    print "\n<!-- #STOP TABLES=\"T\" -->\n";
}

#******************* SUB PRINTFORMOPENING *********************************
#* Start the <FORM> tag for the HTML page
#*In starting <FORM> the action should be this file, whose name is 
#*extracted from __FILE__.  This means that the cgi can be renamed freely.
#******************* SUB PRINTFORMOPENING **********************************

sub printformopening {
    local ($file);
    $file=__FILE__;
    ($file) = ($file =~ /^.*\/(.*)$/);
    print "<FORM METHOD=\"post\" ACTION=\"$file\">\n";
}

#******************* SUB PRINTOPENING *********************************
#*Open the HTML page.  Titles come from:
#*  1.  The argument
#*  2.  The global variable $title
#*  3.  The global constant $DEFAULTTITLE
#*  4.  $title="no title"
#******************* SUB PRINTOPENING **********************************

sub printopening {
    local ($pagetitle)=@_;
    if (!$pagetitle){$pagetitle=$title;}
    if (!$pagetitle){$pagetitle=$DEFAULTTITLE;}
    if (!$pagetitle){$pagetitle="no title";}
    print "<HTML><HEAD><TITLE>$pagetitle</TITLE></HEAD>\n";
    print "<BODY BACKGROUND=\"mtibk.gif\"><CENTER>\n";
}

#******************* SUB PRINTCLOSING *********************************
#*Post all data and close the page
#******************* SUB PRINTCLOSING **********************************

sub printclosing {
    print "</CENTER></BODY></HTML>\n";
}

#******************* SUB PRINTFORMDATA *********************************
#* Print all form data as hidden input
#******************* SUB PRINTFORMDATA **********************************

sub printformdata {
    print "
<INPUT TYPE=\"hidden\" NAME=\"score\" VALUE=\"$score\">
<INPUT TYPE=\"hidden\" NAME=\"stage\" VALUE=\"$stage\">
<INPUT TYPE=\"hidden\" NAME=\"substage\" VALUE=\"$substage\">
<INPUT TYPE=\"hidden\" NAME=\"GL\" VALUE=\"$GL\">
<INPUT TYPE=\"hidden\" NAME=\"cookie\" VALUE=\"$cookie\">
";
    print "<INPUT TYPE=\"hidden\" NAME=\"clues\" VALUE=\""
	. join(" ",@clues) . "\"><BR></FORM>";
}

#******************* SUB IMAGETAG *********************************
#*Passed a filename, returns an appropriate image tag for the 
#*data stored in that file.  Assumes that the file is of the
#*form filename<TAB>width<TAB>height\n
#******************* SUB IMAGETAG **********************************

sub imagetag 
{

    local ( $dataname ) = @_;
    local ( $result );
    
    local ( $iname, $iwidth, $iheight ) = 
	split ( "\t" , &getdata( "$dataname" ) );

    ($idir , $iname) = $iname =~ /(.*\/)(.*)/;

    return 
	"<IMG SRC=\"$DATADIR$idir$GL.$iname\" " .
	    "WIDTH=\"$iwidth\" HEIGHT=\"$iheight\" ".
		"ALT=\"$iname\" BORDER=0> <BR CLEAR>";
}

#******************* SUB GETDATA *********************************
#*Given a string, use the string as a filename; read the entire file,
#*and return the concatenated contents. 
#****************** SUB GETDATA **********************************

sub getdata {
    local ($name)=@_;
    $name=$DATADIR . $name;
    local ($data);

    #
    # Read the entire file into $data
    #

    open(IN,$name);

    $data="";
    while (<IN>){
	$data.=$_;
    }

    close(IN);

    #
    # Remove trailing \n, if any
    #

    if ($data =~ /\n$/)
    {
	chop($data);
    }

    #
    # return the result
    #
    
    return $data;
}

#******************* SUB TOKENIZE *********************************
#*Takes a filename (required) and an x and y coordinate pair (optional).
#*If $x and $y are omitted, they are read from the global $in{"x"} and
#*$in{"y"}.  Returns the token (string) corresponding to the last
#*rectange in which the point (x,y) falls in the imagemap file, or 
#*"" on failure.  File format is a series of tab-separated lines
#*of the form 
#*TOKEN  LEFT  TOP  RIGHT  BOTTOM
#*(lines that begin with # are ignored.)
#****************** SUB TOKENIZE **********************************

sub tokenize {
    # Initialize parameters
    local ($im,$x,$y)=@_;
    
    # Prepare the coordinates:
    if(!$x || !$y){
	$x=$in{"map.x"};
	$y=$in{"map.y"};
    }

    # Other local variables (to take values read from file

    local ($intoken,$left,$top,$right,$bottom,$result,@im);

    # Find the token in $im

    @im=split("\n",$im);

    foreach (@im)
    {
	next if /^\s*#/;      # Comments begin with a #
	next if /^\s*$/;       # Ignore blank lines

	( $intoken , @stuff )=split("\t");

	#
	# Handle polygons
	#

	if ( $stuff[0] =~ /poly/ )
	{
	    shift ( @stuff );
	    if ( &in_polygon ( $x, $y, @stuff) )
	    {
		$result = $intoken;
	    }
	}

	# Handle Circles -- untested!
	
	elsif ( $stuff[0] =~ /circ/ )
	{
	    shift ( @stuff );
	    if ( &in_circle ( $x, $y, @stuff ) )
	    {
		$result = $intoken;
	    }
	}
	
	# Not a polygon -- must be a rectangle
	

	#
	# If I had time to test, this would first check for the prefix
	# "rect" in the same way that circle checks for "circ" and 
	# polygon checks for poly.
	#

	( $left, $top, $right, $bottom ) = @stuff;
	if($left<=$x && $x<=$right && $top<=$y && $y<=$bottom)
	{
	    $result=$intoken;
	}
    }
    
    #
    # $result is the $intoken we found, if any.
    #

    return $result;
}

#
# Scooped off a much more general varmap routine, and modified
# somewhat.  Takes an x-value, a y-value, and a list of 
# vertices.
#

#
# Here are the fucking steps for the polygon container code.
# The following operations are on each line segment.
# If the_x is not with the bounds of the_x1 and the_x2,
# skip to the next iteration.
# Measure the slope and y-intersept of the line.
# Input the_x to the equation of the line.
# If the resulting value is <= the_y, add 1 to the_cross_count.
# end loop
# If ! ( $the_cross_count % 2 ) return $the_url.s
#

sub in_polygon {
    local ( $the_x, $the_y, @the_details ) = @_;
    local ( $the_min, $the_max, );
    local ( $the_v1, $the_v2 );
    local ( @the_verts0, @the_verts1 );

    #
    # Initialize the maxima and minima.
    # 

    ( $the_min[0], $the_max[0] ) = ( 0, 4096 );
    ( $the_min[1], $the_max[1] ) = ( 0, 4096 );
    
    # 
    # Read in the verticies and determine the x and y bounding box.
    # 

    for ( $i = 0 ; $i < $#the_details ; $i += 2 ) {
	$the_verts0[ $i / 2 ] = $the_details[ $i ];
	$the_verts1[ $i / 2 ] = $the_details[ $i + 1 ];
	
	if ( $the_x == $the_verts0[$i] && $the_y == $the_verts1[$i] ) 
	{
	    # 
	    # The point is a vertex.
	    # 
	    return 1;
	}
    }
    
    $the_cross_count = 0;
    
    for ( $i = 0 ; $i <= $#the_verts0 ; $i ++ ) {
	if (   ( $the_verts0[$i] == $the_x )
	    && ( $the_verts1[$i] <  $the_y ) )
	{
	    $the_cross_count ++;
	}
	
	$the_prev = ( $i + $#the_verts0 ) % ( $#the_verts0 + 1 );
	
	( $the_v1[0], $the_v1[1] )
	    = ( $the_verts0[$the_prev], $the_verts1[$the_prev] );
	( $the_v2[0], $the_v2[1] ) = ( $the_verts0[$i], $the_verts1[$i] );
	
	# Go to the next iteration if the slope is undefined.
	# Increment cross_count if y is on the line.
	# 
	$the_max_x = ( $the_v1[0] > $the_v2[0] ) ? $the_v1[0] : $the_v2[0];
	$the_min_x = ( $the_v1[0] < $the_v2[0] ) ? $the_v1[0] : $the_v2[0];
	
	$the_max_y = ( $the_v1[1] > $the_v2[1] ) ? $the_v1[1] : $the_v2[1];
	$the_min_y = ( $the_v1[1] < $the_v2[1] ) ? $the_v1[1] : $the_v2[1];
	
	# 
	# Point is on a vertical line, so it is in the polygon.
	# 
	if ( $the_min_x == $the_max_x ) 
	{
	    if ( ( $the_x == $the_min_x )
		&& ( $the_y <= $the_max_y )
		&& ( $the_y >= $the_min_y ) ) 
	    {
		return 1;
	    }
	    next;
	}
	
	# 
	# Point is on a horizontal line, so it is in the polygon.
	# 
	if ( $the_min_y == $the_max_y ) {
	    if ( ( $the_y == $the_min_y )
		&& ( $the_x <= $the_max_x )
		&& ( $the_x >= $the_min_x ) ) {
		return 1;
	    }
	}
	
	if ( ( $the_x > $the_max_x ) || ( $the_x < $the_min_x ) ) {
	    next;
	}
	
	$the_slope = ( $the_v2[1] - $the_v1[1] ) / ( $the_v2[0] - $the_v1[0] );
	
	$the_y_int = $the_v1[1] - $the_slope * $the_v1[0];
	
	$the_final_y = $the_slope * $the_x + $the_y_int;
	
	if ( $the_y >= $the_final_y ) {
	    $the_cross_count++;
	    next;
	}
    }
    
    if ( ( $the_cross_count != 0 ) && ( $the_cross_count % 2 != 0 ) ) 
    {
	return 1;
    }
    else 
    {
	return 0;
    }
}

#
# UNTESTED! 
#

sub in_circle 
{
    local ( $the_x, $the_y, $the_center_x, $the_center_y , $the_radius  ) 
	= @_;
    
    if ( ( $the_center_x - $the_x )**2
	+ ( $the_center_y - $the_y )**2 <= $the_radius**2 ) 
    {
	return 1;
    } 
    else 
    {
	return 0;
    }
}


sub send_redirect 

{
    local ( $the_url ) = @_;

    $CRLF = "\r\n"; 

    print join($CRLF,"Status: 302 Found",
                "Location: ${the_url}",
                "URI: $the_url",
                "Content-type: text/html"). # patches a bug in some servers
                    "${CRLF}${CRLF}";
}

#
# The following prints the Disney standard legal disclaimer footer
#

sub printlegal

{
    print <<"--Legaleeze--";
<FONT SIZE="-2">
<P>

(c) Disney. All rights reserved.<br>
Muppet characters (c) Jim Henson Productions, Inc.<BR>

<P>

Jim Henson, the Jim Henson signature logo, MUPPETS, <BR>
MUPPET, and all character names and likenesses are <BR>
Trademarks of Jim Henson Productions, Inc.<BR>
All rights reserved.

</FONT>
--Legaleeze--

}
