#!/usr/local/bin/perl

#
# 1995-May-07: seth: Edit the .htaccess and .htgroup files.
#
# htacedit <htapasswd-path> <htgroup-path> <group> <user> <pass>
#
# Returns:
#   0 on success
#   1 if user can't be added b/c a dup name exists.
#   2 if cmd line args are no good
#   3 if permission denied or files not found
#

#
# Some files we're interested in
#

$HTPASSEXEC = "/usr/local/etc/httpd/support/htpasswd";

#
# Do BSD style local file locking.  Be carefull as these are advisory only
#

$LOCK_SH = 1;
$LOCK_EX = 2;
$LOCK_NB = 4;
$LOCK_UN = 8;

#
# Here we go ...
#

sub main

{
    local ($result);

    #
    # Verify params
    #

    exit 2 if ($#ARGV == 6);
    ( $htpasswd, $htgroup, $group, $user, $pass ) = @ARGV;

    #
    # Open and lock the shared permissions files
    #

    open (HTGROUP, "+<$htgroup") || exit 3;
    open (HTPASSWD, "+<$htpasswd") || exit 3;

    &lock(HTGROUP);
    &lock(HTPASSWD);

    #
    # Call subr to do the work
    #

    $result = &add ($user, $pass);

    #
    # Release the locks
    #

    &unlock(HTGROUP);
    &unlock(HTPASSWD);

    close (HTGROUP);
    close (HTPASSWD);

    exit $result;
}

$LOCK_SH = 1;
$LOCK_EX = 2;
$LOCK_NB = 4;
$LOCK_UN = 8;

sub lock

{
    local ( $file ) = @_;
    flock ( $file, $LOCK_EX );
}

sub unlock

{
    local ( $file ) = @_;
    flock ( $file, $LOCK_UN );
}

sub add 

{
    local ( $user, $pass ) = @_;
    local ( $salt, @s );

    return 1 unless &unique_user ($user);

    #
    # Generate some salt for crypt
    #

    srand(time|$$);
    $i = 0;
    while ($i < 2){
	$s[$i] = pack("C", int(rand(93)) + 33);
	$s[$i] =~ /[A-Za-z0-9]/ && $i++;
    }
    $salt = $s[0] . $s[1];

    #
    # Seek to the end of the group and passwd files
    #

    seek (HTGROUP, 0, 2);
    seek (HTPASSWD, 0, 2);

    #
    # Append the new entries to the files
    #

    print HTGROUP "$group", ": ", "$user", "\n";
    print HTPASSWD "$user", ":", crypt ($pass, $salt), "\n";

    0;
}

sub unique_user

{
    local ( $user ) = @_;

    #
    # Make sure we're at the top of the file
    #

    seek (HTPASSWD, 0, 0);

    #
    # Scan the user name field for the user we want
    #

    while (<HTPASSWD>){
	/^${user}:.*/ && return 0;
    }

    1;
}

&main;
0;


