#!/usr/local/bin/perl

# errorcgi.pl by Sam Nelson, January 26, 1996, v1.0
# Program written for SmallWorld Software
#
# Implements the function &error, generating an html page
# as output.  May be used for all error routines in a site.
# 
# Limitations:  Error produces an entire page to itself, so it must
# be called after the cgiHeader call, but before any output has been
# produced.  Furthermore, no more output may be produced after a call 
# to exit.

$DEBUG=1;          # Set to 0 to turn off debuggin'

# Exit/error codes
$SUCCESS=0;        # No error
$BADENGINE=1;      # Bad Trivia engine
$USERERROR=2;      # The calling function delivers the message.
$BADARGS=3;        # Arguments to the CGI not passed correctly
$BADERRORCODE=4;   # Invalid error code passed to &error()
$BADCOOKIEFILE=5;  # Unable to access the cookie file
$BADPAGE=6;        # The requested html file could not be opened.
$OUTOFQUESTIONS=7; # Out of questions
$BADUIDFILE=8;     # Unable to access user database
$BADUIDVALUE=9;    # Unable to find needed data in the user database
$BADEMAIL=10;      # User entered a malformed email address
$BADRANK=11;       # Unknown rank for a treasure island game
$BADSCORE=12;      # Bad score passed to a cgi
$BADWORD=13;       # Bad word used in nickname

# $ERRORMESSAGE[$USERERROR]=
# handled specially by error routine
$ERRORMESSAGE[$SUCCESS]=
"No error.";
$ERRORMESSAGE[$BADENGINE]=
"Unable to execute the trivia engine.";
$ERRORMESSAGE[$BADARGS]=
"Bad arguments passed to the CGI.";
$ERRORMESSAGE[$BADERRORCODE]=
"Unrecognized error code.";
$ERRORMESSAGE[$BADCOOKIEFILE]=
"Unable to get needed data from your temporary file.";
$ERRORMESSAGE[$BADPAGE]=
"Unable to access the requested html document.";
$ERRORMESSAGE[$OUTOFQUESTIONS]=
"You have completed all the questions in our bank.";
$ERRORMESSAGE[$BADUIDFILE]=
"Unable to access the user database.";
$ERRORMESSAGE[$BADUIDVALUE]=
"Unable to find needed data in the user database.";
$ERRORMESSAGE[$BADEMAIL]=
"Your email address is invalid.  Please check to make sure you entered
it correctly.";
$ERRORMESSAGE[$BADRANK]=
"Don't know how to find the rank for the given score.";
$ERRORMESSAGE[$BADSCORE]=
"A bad score was passed to a program that needs to know your score.";
$ERRORMESSAGE[$BADWORD]=
"Your nickname is invalid.  Please make sure that you entered
it correctly.";

#************* SUB ERROR ***********************************
#* Output a page explaining why we cannot do what the user
#* wants.  The calling function either delivers one of the
#* error codes given above, or sends the code $USERERROR
#* as the first argument and sends their own error messgae
#* as the second argument.  (If the error code is anything
#* but $USERERROR, the system error message will override
#* any error message supplied.
#************* SUB ERROR ***********************************

sub error {
    # Get args to error
    local($code,$usermessage,$message);
    ($code,$usermessage)=@_;

    # decipher the error message
    if ($code eq $USERERROR){$message=$usermessage;}
    else {$message=$ERRORMESSAGE[$code];}
    if (!defined($message) || ($message eq "")){
	$message=$ERRORMESSAGE[$BADERRORCODE];
    }

    # Print error page and exit
    print "<HTML><HEAD><TITLE>An Error Occurred</TITLE></HEAD>";
    print "<BODY BACKGROUND=\"mtibk.gif\">";
    print "<CENTER><H1>An Error Occurred</H1></CENTER><HR>";
    print "We were unable to complete your request for the following reason:";
    print "<P><STRONG>$message</STRONG>";
    print "<HR></BODY></HTML>";
    exit($code);
}
