#!/usr/local/bin/perl

# leader.pl by Sam Nelson, February 22, 1996, v1.0
# Program written for SmallWorld Software
#
# A program to implement basic Leader's List functions

require './errorcgi.pl';

#
# Constants
#
$NUMLEADERS=100;
$INITIALSCORE=1000000;
$SCOREDIR='./';
$SCOREFILE='score.dat';

$BADWORDSDIR = './';
$BADWORDSFILE = 'badwords.dat';

1;

#************* SUB SCORE_TO_RANK **********************
#*Takes a score.  Returns the (numeric) rank associated with
#*that score.
#************* SUB SCORE_TO_RANK ***********************

sub score_to_rank {
    local ($score)=(@_);
    local ($rank)=("");
    open(SCORE,$SCOREDIR . $SCOREFILE) || &error($BADFILE);
    while(<SCORE>)
    {
	next if /#/;
	next if !$_;
	($new_rank,$new_score)=split("\t",$_);
	if ( !$rank && ($score>$new_score) ){ $rank=$new_rank; }
    }
    close(<SCORE>);
    return($rank);
}

#************* SUB CHECKBADWORDS **********************
#*Takes a score.  Returns the (numeric) rank associated with
#*that score.
#************* SUB CHECKBADWORDS ***********************

sub checkbadwords
{
    local ($input) = @_;
    local ($word , $bigword , $count , $same);

    #
    # Nickname can't contain anything but word characters
    #

    if ($input !~ /^\w+$/) 
    {
	return 1;
    }

    #
    # Load the bad words file
    #

    open( BADWORDS , $BADWORDSDIR . $BADWORDSFILE ) || &error($BADFILE);

    $count = 0;
    while (<BADWORDS>)
    {
	chop;
	next if /#/;
	next if !$_;
	@badwords [ $count++ ] = $_;
    }

    #
    # Now check against each bad word
    #
    
    foreach $word (@badwords)
    {
	#
	# $bigword is $word with each letter separated by (.*)
	#

	@bigword = split ( // , $word );
	$bigword = join ( '(.*)' , @bigword );

	#
	# Move on if we can tell immediately we're ok
	#

	next if $input !~ /$bigword/i;

	#
	# Now we suspect something of the form F_U__C___K.  We're
	# ok if one of the things between the letters (separators):
	# -- exists,
	# -- is not entirely underscores, and
	# -- is not the same as the first separator
	# -- is not a case-insensitive repetition of that letter

	#
	# First, generate the dividers (from the pattern match above
	#

	for ($count = 1 ; $count < 10 ; $count ++ )
	{
	    $break[$count] = eval ( "\$$count" );
	}

	#
	# Assume that the dividers are the same
	#

	$same = 1;

	# 
	# But they're different if they exist, aren't all underscores,
	# and are different strings from the first one.
	#

	for ($count = 0 ; $count < 10 ; $count ++ )
	{
	    if (
		$break[$count] && 
		($break[$count] ne $1) &&
		($break[$count] !~ /^$bigword[$count]+$/i) &&
		($break[$count] !~ /^_*$/)
		)
	    {
		$same = 0;
	    }
	}

	#
	# if the separators are the same, veto the word.
	#

	if ($same)
	{
	    return 1;
	}
    }

    #
    # No bad words found
    #
    
    return undef;
}
