#!/usr/local/bin/perl

# muppet_list.pl by Sam Nelson, February 15, 1996, v1.0
# Program written for SmallWorld Software
#
# A program to generate mass mailings to users on the
# muppet_list or clueless_list.
#
# Requires a $uid file indexed by email address and
# readable by the uidcgi.pl library.  
#
# The following files must exist in $LISTDIR (./):
# -- clueless_list.txt 
#    The text of the current clueless mailing
# -- muppet_list.<month>.txt
#    The text of the muppet list mailing for the current month
#

#
# $USAGE
#
$USAGE="mailer.pl [-c] [-m]
The -c option sends the file clueless_list.txt to all users
on the clueless_list mailing list.

The -m option sends the file muppet_list.<month>.txt to all
users on the muppet_list mailing list.  <month> represents
the current month, in 3-letter, capitalized form (\"Jan\", \"Feb\",
etc.)

Users are assumed to be listed in the userid file specified in 
uidcgi.pl.  The uid is assumed to be an email address, and 
users with a value of \"TRUE\" for the variables \"clueless_list\" and
\"muppet_list\" are assumed to be on the mailing lists.
";

#
# requires and constants
#
require 'uidcgi.pl';
require 'getopts.pl';
require 'ctime.pl';

$LISTDIR='./';

#
# Get operands
#
&Getopts('cm');

if (!$opt_c && !$opt_m){
    print "USAGE:  $USAGE";
    exit(0);
    die($USAGE);
}

#
# Execute the clueless mailing
#
if ($opt_c){
    $messagefile=$LISTDIR . "clueless_list.txt";
    %list=&readuidfile;
    foreach $uid (keys(%list)){
	%data=split("\t",$list{$uid});
	if ($data{"clueless_list"} eq "TRUE"){
	    $to=$data{"first"} . " " . $data{"last"};
	    $from="Clueless The Pirate <disney_online_info@online.disney.com>";
	    $subject="Your biweekly mailing from Clueless!";
	    $address=$uid;
	    $sender="disney_online_info@online.disney.com";
	    
	    if (
		($address !~ /aol\.com/) &&
		($address !~ /compuserve\.com/) &&
		($address !~ /prodigy\.com/) &&
		($address !~ /applelink\.apple\.com/) &&
		($address !~ /eworld\.com/)
		)
	    {
		$imagefile = "Clueless.gif";
	    }
	    else
	    {
		$imagefile = "Clueless.txt";
	    }

	    &sendmessageandimage(
				 $to,$from,$subject,$address,$sender,
				 $messagefile,$imagefile
				 );

	    print "sent mail to $address\n";
	}
    }
}

#
# Execute the muppet mailing
#
if ($opt_m){
    $month=(Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec)
	[(localtime)[4]];
    $messagefile=$LISTDIR . "muppet_list.$month.txt";
    %list=&readuidfile;
    foreach $uid (keys(%list)){
	%data=split("\t",$list{$uid});
	if ($data{"muppet_list"}){
	    $to=$data{"first"} . " " . $data{"last"};
	    $from="$data{\"muppet_list\"} <disney_online_info@online.disney.com>";
	    $subject="Your monthly muppet mailing!";
	    $address=$uid;
	    $sender="disney_online_info@online.disney.com";

	    if (
		($address !~ /aol\.com/) &&
		($address !~ /compuserve\.com/) &&
		($address !~ /prodigy\.com/) &&
		($address !~ /applelink\.apeple\.com/) &&
		($address !~ /eworld\.com/)
		)
	    {
		$imagefile="$data{\"muppet_list\"}.gif";	    
	    }
	    else
	    {
		$imagefile="$data{\"muppet_list\"}.txt";
	    }
	    
	    &sendmessageandimage(
				 $to,$from,$subject,$address,$sender,
				 $messagefile,$imagefile
				 );

	    print "sent mail to $address\n";
	}
    }

}

#
# Out, damn spot!  Out I say
#
exit(0);

#************************************************************
#*Send a mail message, including a message file and an image
#*file.  Sorry 'bout the long and ugly line of parameters,
#*all required, alas.  
#************************************************************

sub sendmessageandimage{
    local($to,$from,$subject,$addres,$sender,$messagefile,$imagefile)=@_;
    local($sendmail);

    #
    # CONSTANTS!
    #

    $sendmail="/usr/lib/sendmail";

    local($DIVIDER);
    $DIVIDER="========================_34892636==_";
    
    local($MESSAGEHEADER);
    $MESSAGEHEADER="Content-Type: text/plain; charset=\"us-ascii\"";
    
    local($GIFHEADER);
    $GIFHEADER="Content-Type: image/gif; name=\"$imagefile\";
 x-mac-type=\"47494666\"; x-mac-creator=\"4A565752\"
Content-Transfer-Encoding: base64
Content-Disposition: attachment; filename=\"$imagefile\"";
    
    local($MAILHEADER);
    $MAILHEADER="To:  \"$to\"
Mime-Version: 1.0
Content-Type: multipart/mixed; boundary=\"$DIVIDER\"
From: $from
Reply-To: $from
Subject: $subject";
    
    # 
    # Open the sendmail pipe
    #
    
    open(SENDMAIL,"| $sendmail -f $sender $address") || 
	die("Must have access to sendmail");
    
    select(SENDMAIL);
    
    #
    # Print the header
    #

    print "$MAILHEADER\n\n\n--$DIVIDER\n\n";
    print "$MESSAGEHEADER\n\n";
    
    #
    # Open the messagefile, dump it, and close it
    #

    open(MESSAGE,"$messagefile") || 
	die("unable to open message file $messagefile");

    while(<MESSAGE>)
    {
	print $_;
    }
    close(MESSAGE);
	
    # 
    # If the image file ends in .gif, add it as an attachment
    #

    if ($imagefile =~ /\.gif$/)
    {
	print "\n\n--$DIVIDER\n$GIFHEADER\n\n";

	open(IMG,"./encdec -b -e < $imagefile |") || 
	    die("Couldn't open image file $imagefile");
	while(<IMG>){
	    print $_;
	}
	close(IMG);
    }
    
    #
    # Otherwise if it ends in .txt, dump it into the message
    #

    elsif ($imagefile =~ /\.txt$/)
    {
	open(IMG,"$imagefile") ||
	    die("unable to open image file $imagefile");
	
	while (<IMG>)
	{
	    print $_;
	}

	close(IMG);
    }

    #
    # Otherwise, we were passed a bad $imagefile.
    #

    else
    {
	die ( "Bad image file type $imagefile" );
    }

    # 
    # Finish up the mail message
    #

    print "--$DIVIDER\n\n";
    close(SENDMAIL);
    select(STDOUT);
}
